import java.awt.Color;
import java.awt.Graphics2D;

public class Board {
	private boolean[][] board = new boolean[17][17];
	private boolean[][] used = new boolean[17][17];
	private Wall[][] wall = new Wall[2][Const.WALL_AMOUNT];
	private int[] rest = new int[2];
	private Token[] token = new Token[2];

	public Board() {
		for (int i = 0; i < 17; i++) {
			for (int j = 0; j < 17; j++) {
				board[i][j] = false;
			}
		}
		for (int i = 0; i < 2; i++) {
			rest[i] = Const.WALL_AMOUNT;
			for (int j = 0; j < Const.WALL_AMOUNT; j++) {
				wall[i][j] = new Wall(i, j);
			}
		}

		token[0] = new Token(0, 4, 0);
		token[1] = new Token(8, 4, 1);
	}

	public void setData(Data data, int player) {
		for (int i = 0; i < 2; i++) {
			data.tokenX[i] = token[i ^ player].getX();
			data.tokenY[i] = token[i ^ player].getY();
			data.restWalls[i] = rest[i ^ player];
		}
		for (int i = 0; i < 17; i++) {
			int i2 = i;
			if (player == 1) {
				i2 = 16 - i;
			}
			for (int j = 0; j < 17; j++) {
				data.board[i][j] = board[i2][j];
			}
		}
	}

	public boolean moveToken(int player, int direction) {
		if (direction < 0 || direction > 3) {
			return false;
		}
		int x = token[player].getX();
		int y = token[player].getY();
		int nx = x + Const.DX[direction];
		int ny = y + Const.DY[direction];
		if (nx < 0 || ny < 0 || nx >= Const.BOARD_SIZE
				|| ny >= Const.BOARD_SIZE || board[x + nx][y + ny]) {
			return false;
		}
		token[player].move(direction);
		return true;
	}

	public boolean putWall(int player, int x, int y, int direction) {
		if (rest[player] <= 0 || x < 0 || y < 0 || x > 7 || y > 7
				|| direction < 0 || direction > 1) {
			return false;
		}

		if (direction == Const.HORIZONTAL) {
			for (int i = 0; i < 3; i++) {
				if (board[x * 2 + i][y * 2 + 1]) {
					return false;
				}
				board[x * 2 + i][y * 2 + 1] = true;
			}
		} else {
			for (int i = 0; i < 3; i++) {
				if (board[x * 2 + 1][y * 2 + i]) {
					return false;
				}
				board[x * 2 + 1][y * 2 + i] = true;
			}
		}
		if (cantGoal(0) || cantGoal(1)) {
			return false;
		}
		rest[player]--;
		wall[player][rest[player]].setLocation(x, y, direction);
		return true;
	}

	private boolean cantGoal(int player) {
		for (int i = 0; i < 17; i++) {
			for (int j = 0; j < 17; j++) {
				used[i][j] = false;
			}
		}
		return dfs(token[player].getX(), token[player].getY(), (player ^ 1) * 8);
	}

	private dfs(int x, int y, int goal) {

	}

	public void draw(Graphics2D g) {
		g.setColor(new Color(Const.BOARD_COLOR));
		g.fillRect(108, 38, 424, 424);
		for (int i = 0; i < Const.BOARD_SIZE; i++) {
			for (int j = 0; j < Const.BOARD_SIZE; j++) {
				g.setColor(new Color(Const.CELL_COLOR));
				g.fillRect(118 + i * 46, 48 + j * 46, 36, 36);
			}
		}
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < Const.WALL_AMOUNT; j++) {
				wall[i][j].draw(g);
			}
			token[i].draw(g, (token[0].getX() == token[1].getX() && token[0]
					.getY() == token[1].getY()));
		}
	}
}
