import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;

import javax.swing.JFrame;

public class Main extends JFrame {
	private static final long serialVersionUID = 1L;

	Image back;

	private Board board;
	private AI[] ai = new AI[2];
	private int turn, wait, winner;
	private boolean end;
	private Data data;

	public static void main(String args[]) {
		new Main();
	}

	public Main() {
		super("Quoridor");
		init();
		ThreadClass threadcls = new ThreadClass();
		Thread thread = new Thread(threadcls);
		thread.start();
		setSize(Const.WINDOW_WIDTH, Const.WINDOW_HEIGHT);
		setBackground(new Color(Const.BG_COLOR));
		setVisible(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private void init() {
		board = new Board();
		ai[0] = new SampleAI(0);
		ai[1] = new SampleAI(1);
		data = new Data();
		turn = 0;
		wait = 30;
		end = false;
		winner = 0;
	}

	class ThreadClass implements Runnable {
		public void run() {
			while (true) {
				if (wait <= 0 && !end) {
					board.setData(data, turn % 2);

					boolean foul = false;
					Move res = ai[turn % 2].move(data);
					System.out.printf("Turn %d : %d %d %d %d\n", turn,
							res.mode, res.direction, res.x, res.y);
					if (res.mode == -1) {
						foul = true;
					} else if (res.mode == Const.TOKEN) {
						if (turn % 2 == 1 && res.direction <= 1) {
							res.direction ^= 1;
						}
						if (!board.moveToken(turn % 2, res.direction)) {
							foul = true;
						}
					} else {
						if (turn % 2 == 1) {
							res.x = 7 - res.x;
						}
						if (!board.putWall(turn % 2, res.x, res.y,
								res.direction)) {
							foul = true;
						}
					}

					if (foul) {
						winner = (turn % 2) ^ 1;
						System.out.println(winner);
						end = true;
					} else if (goal()) {
						winner = turn % 2;
						end = true;
					} else {
						turn++;
						wait = Const.WAIT;
					}
				} else {
					wait--;
				}

				repaint();
				try {
					Thread.sleep(30);
				} catch (InterruptedException e) {
				}
			}
		}
	}

	private boolean goal() {
		int x = board.getTokenX(turn % 2);
		int gx = (turn % 2 == 0 ? 8 : 0);
		return x == gx;
	}

	public void paint(Graphics g) {
		back = createImage(getWidth(), getHeight());
		Graphics2D g2 = (Graphics2D) back.getGraphics();
		g2.clearRect(0, 0, getWidth(), getHeight());
		board.draw(g2);
		if (end) {
			g2.setColor(new Color(1.0f, 0.0f, 0.0f, 0.5f));
			g2.fillRect(0, 0, getWidth(), getHeight());
		}
		g.drawImage(back, 0, 0, this);
	}
}
