import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;

import javax.swing.JFrame;

public class Main extends JFrame implements MouseMotionListener {
	private static final long serialVersionUID = 1L;

	Image back;

	private Board board;
	private AI[] ai = new AI[2];
	private int turn, wait, winner;
	private boolean end;
	private Data data;

	private int mouseX = 0;
	private int mouseY = 0;

	public static void main(String args[]) {
		new Main();
	}

	public Main() {
		super("Quoridor");
		init();
		ThreadClass threadcls = new ThreadClass();
		Thread thread = new Thread(threadcls);
		thread.start();
		setSize(Const.WINDOW_WIDTH, Const.WINDOW_HEIGHT);
		setBackground(new Color(Const.BG_COLOR));
		setVisible(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		addMouseMotionListener(this);
		addMouseListener(this);
	}

	private void init() {
		board = new Board();

		// AI
		if (Const.IsHuman[0]) {
			ai[0] = new Human(0);
		} else {
			ai[0] = new SampleAI(0);
		}

		if (Const.IsHuman[1]) {
			ai[1] = new Human(1);
		} else {
			ai[1] = new SampleAI(1);
		}
		//

		data = new Data();
		turn = 0;
		wait = 30;
		end = false;
		winner = 0;
	}

	class ThreadClass implements Runnable {
		public void run() {
			while (true) {
				if (wait <= 0 && !end) {
					board.setData(data, turn % 2);

					boolean foul = false;
					Move res = ai[turn % 2].move(data);
					if (res.mode != -1)
						System.out.printf("Turn %d : %d %d %d %d\n", turn,
								res.mode, res.direction, res.x, res.y);
					if (res.mode == -1) {
					} else if (res.mode == Const.TOKEN) {
						if (turn % 2 == 1 && res.direction <= 1) {
							res.direction ^= 1;
						}
						if (!board.moveToken(turn % 2, res.direction)) {
							foul = true;
						}
					} else {
						if (turn % 2 == 1) {
							res.x = 7 - res.x;
						}
						if (!board.putWall(turn % 2, res.x, res.y,
								res.direction)) {
							foul = true;
						}
					}

					if (foul) {
						winner = (turn % 2) ^ 1;
						System.out.println(winner);
						end = true;
					} else if (goal()) {
						winner = turn % 2;
						end = true;
					} else if (res.mode != -1) {
						turn++;
						wait = Const.WAIT;
					}
				} else {
					wait--;
				}

				repaint();
				try {
					Thread.sleep(10);
				} catch (InterruptedException e) {
				}
			}
		}
	}

	public void mouseMoved(MouseEvent e) {
		mouseX = e.getX();
		mouseY = e.getY();
	}

	public void mouseDragged(MouseEvent e) {
	}

	private boolean goal() {
		int x = board.getTokenX(turn % 2);
		int gx = (turn % 2 == 0 ? 8 : 0);
		return x == gx;
	}

	public void paint(Graphics g) {
		back = createImage(getWidth(), getHeight());
		Graphics2D g2 = (Graphics2D) back.getGraphics();
		g2.clearRect(0, 0, getWidth(), getHeight());
		board.draw(g2);
		if (Const.IsHuman[turn % 2]) {
			drawGhost(g2);
		}
		if (end) {
			g2.setColor(new Color(1, 1, 1, 0.75f));
			g2.fillRect(0, 0, getWidth(), getHeight());
			g2.setColor(new Color(Const.PLAYER_COLOR[winner]));
			g2.setFont(new Font("Arial", Font.BOLD, 64));
			g2.drawString("Player " + (winner + 1) + " wins!!", 100, 240);
		}
		g.drawImage(back, 0, 0, this);
	}

	private void drawGhost(Graphics2D g) {
		board.setData(data, turn % 2);
		int mx = mouseX - 118;
		int my = mouseY - 48;
		int x = mx / 46;
		int y = my / 46;
		int x2 = (turn % 2 == 1 ? 8 - x : x);
		if (mx % 46 < 36 && my % 46 < 36 && x >= 0 && y >= 0 && x < 9 && y < 9) {
			int direction = -1;
			for (int i = 0; i < 4; i++) {
				if (x2 == data.tokenX[0] + Const.DX[i]
						&& y == data.tokenY[0] + Const.DY[i]) {
					direction = i;
				}
			}
			if (direction != -1
					&& data.isValidMove(new Move(Const.TOKEN, direction, 0, 0))) {
				Token ghostToken = new Token(x, y, turn % 2);
				ghostToken.setGhost();
				ghostToken.draw(g, data.tokenX[turn % 2 ^ 1] == x2
						&& data.tokenY[turn % 2 ^ 1] == y);
			}
		}

		if (mx % 46 >= 36 && my % 46 < 36 && x >= 0 && y >= 0 && x < 8 && y < 9) {
			y = Math.min(y, 7);

			if (data.isValidMove(new Move(Const.WALL, Const.VERTICAL, x, y))) {
				Wall ghostWall = new Wall(turn % 2, 0);
				ghostWall.setLocation(x, y, Const.VERTICAL);
				ghostWall.setGhost();
				ghostWall.draw(g);
			}
		}
		y = my / 46;
		if (mx % 46 < 36 && my % 46 >= 36 && x >= 0 && y >= 0 && x < 9 && y < 8) {
			x = Math.min(x, 7);

			if (data.isValidMove(new Move(Const.WALL, Const.HORIZONTAL, x, y))) {
				Wall ghostWall = new Wall(turn % 2, 0);
				ghostWall.setLocation(x, y, Const.HORIZONTAL);
				ghostWall.setGhost();
				ghostWall.draw(g);
			}
		}
	}
}
